# Generated by roxygen2: do not edit by hand

export(CMI)
export(CMIM)
export(DISR)
export(JIM)
export(JMI)
export(JMI3)
export(JMIM)
export(MIM)
export(MRMR)
export(NJMIM)
export(cmiMatrix)
export(cmiScores)
export(dnmiMatrix)
export(hScores)
export(icmiMatrix)
export(impScores)
export(jhScores)
export(jmiMatrix)
export(jmiScores)
export(joinf)
export(kInverse)
export(kTransform)
export(maxCmiScores)
export(maxJmiScores)
export(miMatrix)
export(miScores)
export(minCmiScores)
export(minMaxCmiScores)
export(njmiMatrix)
export(njmiScores)
export(nmiMatrix)
export(triScores)
export(tspTransform)
importFrom(stats,setNames)
importFrom(utils,tail)
useDynLib(praznik, .registration=TRUE)
